@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2018 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -1395,6 +1395,64 @@ ftp_getresp(ftpbuf_t *ftp)
 }
 /* }}} */
 
+int single_send(ftpbuf_t *ftp, php_socket_t s, void *buf, size_t size) {
+#ifdef HAVE_FTP_SSL
+	int err;
+	zend_bool retry = 0;
+	SSL *handle = NULL;
+	php_socket_t fd;
+	size_t sent;
+
+	if (ftp->use_ssl && ftp->fd == s && ftp->ssl_active) {
+		handle = ftp->ssl_handle;
+		fd = ftp->fd;
+	} else if (ftp->use_ssl && ftp->fd != s && ftp->use_ssl_for_data && ftp->data->ssl_active) {
+		handle = ftp->data->ssl_handle;
+		fd = ftp->data->fd;
+	} else {
+		return send(s, buf, size, 0);
+	}
+
+	do {
+		sent = SSL_write(handle, buf, size);
+		err = SSL_get_error(handle, sent);
+
+		switch (err) {
+			case SSL_ERROR_NONE:
+				retry = 0;
+				break;
+
+			case SSL_ERROR_ZERO_RETURN:
+				retry = 0;
+				SSL_shutdown(handle);
+				break;
+
+			case SSL_ERROR_WANT_READ:
+			case SSL_ERROR_WANT_CONNECT: {
+					php_pollfd p;
+					int i;
+
+					p.fd = fd;
+					p.events = POLLOUT;
+					p.revents = 0;
+
+					i = php_poll2(&p, 1, 300);
+
+					retry = i > 0;
+				}
+				break;
+
+			default:
+				php_error_docref(NULL, E_WARNING, "SSL write failed");
+				return -1;
+		}
+	} while (retry);
+	return sent;
+#else
+	return send(s, buf, size, 0);
+#endif
+}
+
 /* {{{ my_send
  */
 int
@@ -1402,13 +1460,6 @@ my_send(ftpbuf_t *ftp, php_socket_t s, void *buf, size_t len)
 {
 	zend_long		size, sent;
     int         n;
-#ifdef HAVE_FTP_SSL
-	int err;
-	zend_bool retry = 0;
-	SSL *handle = NULL;
-	php_socket_t fd;
-#endif
-
 
 	size = len;
 	while (size) {
@@ -1427,56 +1478,7 @@ my_send(ftpbuf_t *ftp, php_socket_t s, void *buf, size_t len)
 			return -1;
 		}
 
-#ifdef HAVE_FTP_SSL
-		if (ftp->use_ssl && ftp->fd == s && ftp->ssl_active) {
-			handle = ftp->ssl_handle;
-			fd = ftp->fd;
-		} else if (ftp->use_ssl && ftp->fd != s && ftp->use_ssl_for_data && ftp->data->ssl_active) {
-			handle = ftp->data->ssl_handle;
-			fd = ftp->data->fd;
-		}
-
-		if (handle) {
-			do {
-				sent = SSL_write(handle, buf, size);
-				err = SSL_get_error(handle, sent);
-
-				switch (err) {
-					case SSL_ERROR_NONE:
-						retry = 0;
-						break;
-
-					case SSL_ERROR_ZERO_RETURN:
-						retry = 0;
-						SSL_shutdown(handle);
-						break;
-
-					case SSL_ERROR_WANT_READ:
-					case SSL_ERROR_WANT_CONNECT: {
-							php_pollfd p;
-							int i;
-
-							p.fd = fd;
-							p.events = POLLOUT;
-							p.revents = 0;
-
-							i = php_poll2(&p, 1, 300);
-
-							retry = i > 0;
-						}
-						break;
-
-					default:
-						php_error_docref(NULL, E_WARNING, "SSL write failed");
-						return -1;
-				}
-			} while (retry);
-		} else {
-#endif
-			sent = send(s, buf, size, 0);
-#ifdef HAVE_FTP_SSL
-		}
-#endif
+		sent = single_send(ftp, s, buf, size);
 		if (sent == -1) {
 			return -1;
 		}
@@ -1929,6 +1931,12 @@ static void ftp_ssl_shutdown(ftpbuf_t *ftp, php_socket_t fd, SSL *ssl_handle) {
 					/* SSL wants a write. Really odd. Let's bail out. */
 					done = 1;
 					break;
+				case SSL_ERROR_SYSCALL:
+					/* most likely the peer closed the connection without
+					   sending a close_notify shutdown alert;
+					   bail out to avoid raising a spurious warning */
+					done = 1;
+					break;
 				default:
 					if ((sslerror = ERR_get_error())) {
 						ERR_error_string_n(sslerror, buf, sizeof(buf));
@@ -2315,12 +2323,3 @@ bail:
 /* }}} */
 
 #endif /* HAVE_FTP */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: sw=4 ts=4 fdm=marker
- * vim<600: sw=4 ts=4
- */
